

#include "GM862_GPRS.h"

GM862_GPRS::GM862_GPRS() {
	priority = PRIORITY;
	allDone = true;
	GPRSLink = false;
	failCount = 0;

	/* Initialize the globals used in ppp driver */

	/* Set the modem global settings (These are the defaults if not included) */
	PPP_BAUDRATE_TO_USE = 115200;
	PPP_SW_RX_FLOW = FALSE;
	PPP_SW_TX_FLOW = FALSE;
	MODEM_AUTOBAUD = "AT+IPR=115200\r\n";
	//	MODEM_RESET = "ATZ&C0&D0&S0#bnd=3+IFC=0,0\r\n";
	MODEM_RESET = "AT&C0&D0&S0#bnd=3+IFC=0,0\r\n";
	MODEM_ANSWER = "ATA\r\n";
	MODEM_HANGSTRING = "ATH0\r\n";
	MODEM_INITSTRING = "AT+CGDCONT=1,\"IP\",\"internet2.voicestream.com\"\r\n";
	sprintf(DIAL_STRING, "ATD*99***1#\r\n");
	MODEM_ENDCMDMODE = "ATO0\r\n";
	MODEM_POSTDIAL = "\r\n";
	MODEM_ATTNSTRING = "+++";

	/* A reasonable set of PPP limits and negotiation settings */
	pppo.Restart_Interval = 3; /* The restart interval for PPP Negotiations */
	pppo.Max_Terminate = 2;
	pppo.Max_Configure = 25;
	pppo.Max_Failure = 25;
	pppo.TX_DESIRED_ACCM = 0x00000000;
	pppo.RX_DESIRED_ACCM = 0x00000000;
	pppo.Chat_Login_disable = TRUE;
	pppo.CHAPenable = FALSE;

	/*------------------------------------------------------------------------------*/

	/* We are dialing through a modem*/
	pppo.SetThisInterfaceAddress = 0; /* We expect the other side to provide an address so we set ours to 0 */
	pppo.SetThatInterfaceAddress = 0;
	pppo.authenticate_func = NULL; /* We don't require the ISP to provide us with Username/password, we dialed him */

	pppo.pUserName = ACCOUNT_NAME; /* The user name password we are providing the ISP if he asks */
	pppo.pPassword = ACCOUNT_PASS;
}

void GM862_GPRS::connectionLoop() {
	while (1) {
		int connectionCounter = 0;

		while (allDone == false) {
			iprintf("\r\nDialing...\r\n");
			int rv = DialPPP(SERIALPORT_TO_USE, &pppo, DIAL_STRING);
			if (rv != ERR_PPP_SUCCESS)
				ShowErrors("Dial Failed", rv);
			else {
				iprintf("Connection Established\r\n");
				EthernetIpGate = 0;
			}

			/*------------------------------------------------------------------------------*/

			/* We have dialed or listened for a receive and the PPP Deamon is running */
			/* Normally your program would do whatever it does.... */
			/* we choose to use the PPP in various arbitrary ways....*/

			if (GetPPPState() != eOpen)

				OSTimeDly(TICKS_PER_SECOND * 5);

			while (GetPPPState() == eOpen && allDone == false) {
				GPRSLink = TRUE;
				OSTimeDly(3 * TICKS_PER_SECOND);
				//iprintf("Active Connection, Secs:%d\r\n",Secs);

			}
			ClosePPPSesion();
			StopPPPDameon();
			GPRSLink = FALSE;
			iprintf("Disconnected from GPRS server.... \r\n");
			connectionCounter++;

		}
		while (allDone == true) {
			iprintf(".");
			OSTimeDly(TICKS_PER_SECOND * 2);
		}
	}
}

void GM862_GPRS::staticconnectionLoop(void * p) {
	GM862_GPRS * po;
	po = (GM862_GPRS*) p;
	po->connectionLoop();
}

void GM862_GPRS::ShowErrors(const char *msg, int errv) {
	switch (errv) {
	case ERR_PPP_ALREADY_OPEN:
		iprintf("%s:ERR_PPP_ALREADY_OPEN\r\n", msg);
		break;
	case ERR_PPP_NO_DIALTONE:
		iprintf("%s:ERR_PPP_NO_DIALTONE\r\n", msg);
		break;
	case ERR_PPP_NO_ANSWER:
		iprintf("%s:ERR_PPP_NO_ANSWER\r\n", msg);
		break;
	case ERR_PPP_BUSY:
		iprintf("%s:ERR_PPP_BUSY\r\n", msg);
		break;
	case ERR_PPP_FAIL:
		iprintf("%s:ERR_PPP_FAIL\r\n", msg);
		break;
	case ERR_PPP_PASSFAIL:
		iprintf("%s:ERR_PPP_PASSFAIL\r\n", msg);
		break;
	case ERR_PPP_LOSTCARRIER:
		iprintf("%s:ERR_PPP_LOSTCARRIER\r\n", msg);
		break;
	case ERR_PPP_NO_MODEM:
		iprintf("%s:ERR_PPP_NO_MODEM\r\n", msg);
		break;
	case ERR_PPP_LCP_FAILED:
		iprintf("%s:Negotiation Loop\r\n", msg);
		break;
	}
}

/*-------------------------------------------------------------------
 DisplayCellErrorCode()
 -------------------------------------------------------------------*/
void GM862_GPRS::DisplayCellErrorCode(int code) {
	char Scode[5];
	for (int i = 0; i < 130; i++)
		if (strcmp(CellErrorCode[(i * 2) + 1], (itoa(code, Scode, 10))) == 0) {
			iprintf("%s\r\n", CellErrorCode[i * 2]);
			return;
		}

	iprintf("Unknown Cell error code [%d]\r\n", code);
	return;
}

void GM862_GPRS::start() {
	OSTaskCreate(GM862_GPRS::staticconnectionLoop, (void *) this,
			(void *) &GPRSTaskStack[USER_TASK_STK_SIZE / 2],
			(void *) GPRSTaskStack, priority);
}

void GM862_GPRS::connect() {
	allDone = false;

}

void GM862_GPRS::disconnect() {
	allDone = true;

}

bool GM862_GPRS::getActiveConnection() {
	return GPRSLink;

}

